from hmac import compare_digest
from hmac import new as hmac_new


def get_hexdigest(key, data, digest):
    return hmac_new(key, data, digest).hexdigest()


def signatures_match(data: bytes, key: bytes, digest: str, signature: str):
    """Test if the signature of data match"""
    return compare_digest(get_hexdigest(key, data, digest), signature)



_callbacks = {}


class NoCallbackError(Exception):
    pass


def register(event, func):
    _callbacks[event] = func


def deregister(event, func):
    del _callbacks[event]


def run(event, headers, payload):
    func = _callbacks.get(event)
    if func is None:
        raise NoCallbackError(f"No callback registered for {event}")
    return func(headers, payload)



from celery import Celery
from flask import Flask

from . import config


def init_celery(app: Flask) -> Celery:
    """Initialize Celery application.

    Args:
        app: Flask app.

    Returns:
        Celery app.
    """
    app.config.update(
        CELERY_BROKER_URL=config.celery_broker,
        CELERY_RESULT_BACKEND=config.celery_backend,
    )
    celery = Celery(
        app.import_name,
        backend=app.config["CELERY_RESULT_BACKEND"],
        broker=app.config["CELERY_BROKER_URL"],
    )
    celery.conf.update(app.config)

    TaskBase = celery.Task

    class ContextTask(TaskBase):
        abstract = True

        def __call__(self, *args, **kwargs):
            with app.app_context():
                return TaskBase.__call__(self, *args, **kwargs)

    celery.Task = ContextTask
    return celery



from flask import Blueprint
from flask import jsonify
from flask import make_response
from flask import request

from .. import __version__
from .. import callbacks
from .. import config
from .utils import signatures_match


webhook = Blueprint("webhook", __name__.split(".", 1)[0])


def _resp(payload, status_code=200):
    return make_response(jsonify(payload), status_code)


@webhook.route("/", methods=["GET"])
def index():
    return _resp({"message": f"ghwh v{__version__}"})


@webhook.route("/webhook", methods=["POST"])
def post_webhook():
    payload = request.get_json()
    headers = request.headers
    event = headers["X-GitHub-Event"]
    guid = headers["X-GitHub-Delivery"]

    signature = headers.get("X-Hub-Signature")
    if config.webhook_secret:
        digest, signature = signature.split("=")
        if not signatures_match(request.data, config.webhook_secret, digest, signature):
            return _resp(
                {"status": "Rejected", "message": "Signature mismatch", "guid": guid},
                400,
            )

    try:
        callbacks.run(event, headers, payload)
    except callbacks.NoCallbackError:
        return _resp(
            {
                "status": "Rejected",
                "message": "No registered callback for the event",
                "guid": guid,
            },
            400,
        )
    return _resp(
        {"status": "Accepted", "message": "Commit message received", "guid": guid}, 202
    )



from ghwh import init_app
from ghwh import init_celery
from ghwh import register


app = init_app()
celery = init_celery(app)


@celery.task()
def handler(headers, payload):
    print(headers)
    print(payload)


def main():
    register("push", handler)
    app.run()



import os

env = os.environ


#: The Celery broker URL used by webhook.
celery_broker = env.get("CELERY_BROKER", "redis://localhost:6379")

#: The Celery backend used by webhook.
celery_backend = env.get("CELERY_BACKEND", "redis://localhost:6379")

#: The secret used for encoding GitHub commit message.
webhook_secret = env.get("WEBHOOK_SECRET", "").encode() or None



import json

from jinja2 import Environment
from jinja2 import PackageLoader


_env = Environment(loader=PackageLoader(__name__, "payloads"))


def render(template, context=None):
    tmpl = _env.get_template(template + ".j2")
    return tmpl.render(**(context or {}))


def load(template, context=None):
    rendered = render(template, context)
    if template.endswith(".json"):
        rendered = json.loads(rendered)
    return rendered



#!/usr/bin/env python
import codecs
import os
import re

from setuptools import find_packages
from setuptools import setup


def find_meta(category, fpath="src/ghwh/__init__.py"):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, fpath), "r") as f:
        package_root_file = f.read()
    matched = re.search(
        r"^__{}__\s+=\s+['\"]([^'\"]*)['\"]".format(category), package_root_file, re.M
    )
    if matched:
        return matched.group(1)
    raise Exception("Meta info string for {} undefined".format(category))


def _parse_requirements(filename):
    with open(filename) as f:
        for line in f:
            line = line.strip()
            if not line or line.startswith("#") or line.startswith("-"):
                continue
            yield line


def requirements(*filenames):
    reqs = []
    for filename in filenames:
        reqs.extend(_parse_requirements(filename))
    return reqs


setup(
    name="ghwh",
    description="GitHub webhook handler app",
    author=find_meta("author"),
    version=find_meta("version"),
    platforms=["Linux"],
    classifiers=[],
    package_data={"ghwh_data": ["payloads/*.j2"]},
    package_dir={"": "src"},
    packages=find_packages("src"),
    scripts=[],
    url="https://github.com/okomestudio/ghwh",
    install_requires=requirements("requirements.txt"),
    extras_require={"dev": requirements("requirements-dev.txt")},
    entry_points={
        "console_scripts": [
            "send_event=ghwh_cli.send_event:main",
            "simple_server=ghwh_cli.simple_server:main",
        ]
    },
)



from argparse import ArgumentParser

from requests import post

from ghwh_data import load


def get_headers(event):
    headers = {
        "X-GitHub-Event": event,
        "X-GitHub-Delivery": "72d3162e-cc78-11e3-81ab-4c9367dc0958",
        "X-Hub-Signature": "sha1=7d38cdd689735b008b3c702edd92eea23791c5f6",
    }
    return headers


def argparser(args=None):
    p = ArgumentParser()
    p.add_argument("event", nargs="?", choices=("push",), default="push")
    p.add_argument("--endpoint", default="http://localhost:5000/webhook")
    return p


def main(args=None):
    parser = argparser()
    args = parser.parse_args(args=args)
    event = args.event
    endpoint = args.endpoint
    headers = get_headers(event)
    payload = load(event + ".json")
    resp = post(endpoint, json=payload, headers=headers)
    print(resp)
